# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest import clients
from tempest.lib import auth
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacEc2CredentialTest(rbac_base.IdentityV3RbacBaseTests,
                                      metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacEc2CredentialTest, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.users_v3_client
        cls.admin_client = cls.os_system_admin
        cls.admin_credentials_client = cls.admin_client.users_v3_client

        # personas in own or other domains
        own_domain_id = cls.persona.credentials.domain_id
        cls.test_client_1, cls.test_user_1, cls.test_project_1 = (
            cls.setup_user_client(domain_id=own_domain_id))
        cls.other_domain_id = cls.admin_client.domains_client.create_domain(
            name=data_utils.rand_name())['domain']['id']
        cls.addClassResourceCleanup(
            cls.admin_client.domains_client.delete_domain, cls.other_domain_id)
        cls.addClassResourceCleanup(
            cls.admin_client.domains_client.update_domain,
            domain_id=cls.other_domain_id, enabled=False)
        cls.test_client_2, cls.test_user_2, cls.test_project_2 = (
            cls.setup_user_client(domain_id=cls.other_domain_id))

    @classmethod
    def setup_user_client(cls, domain_id=None):
        """Set up project user with its own client.

        This is to enable the project user to create its own credential.

        Returns a client object and the user's ID.
        """
        user_dict = {
            'name': data_utils.rand_name('user'),
            'password': data_utils.rand_password(),
        }
        if domain_id:
            user_dict['domain_id'] = domain_id
        user_id = cls.admin_client.users_v3_client.create_user(
            **user_dict)['user']['id']

        def try_cleanup_user():
            # if domain is cleaned up first, user will already be deleted
            try:
                cls.admin_client.users_v3_client.delete_user(user_id)
            except exceptions.NotFound:
                pass

        cls.addClassResourceCleanup(try_cleanup_user)
        project_id = cls.admin_client.projects_client.create_project(
            data_utils.rand_name())['project']['id']
        cls.addClassResourceCleanup(
            cls.admin_client.projects_client.delete_project, project_id)
        member_role_id = cls.admin_client.roles_v3_client.list_roles(
            name='member')['roles'][0]['id']
        cls.admin_client.roles_v3_client.create_user_role_on_project(
            project_id, user_id, member_role_id)
        creds = auth.KeystoneV3Credentials(
            user_id=user_id,
            password=user_dict['password'],
            project_id=project_id)
        auth_provider = clients.get_auth_provider(creds)
        creds = auth_provider.fill_credentials()
        client = clients.Manager(credentials=creds)
        return client, user_id, project_id

    def ec2_credential(self, project_id=None):
        return {
            'tenant_id': project_id or self.project_id
        }

    @abc.abstractmethod
    def test_identity_ec2_create_credential(self):
        """Test identity:ec2_create_credential policy.

        This test must check:
          * whether the persona can create a credential for themself
          * whether the persona can create acredential for another user in
            their own domain
          * whether the persona can create acredential for another user in
            another domain
        """
        pass

    @abc.abstractmethod
    def test_identity_ec2_get_credential(self):
        """Test identity:ec2_get_credential policy.

        This test must check:
          * whether the persona can get their own credential
          * whether the persona can get a credential for a user in another
            domain
          * whether the persona can get a credential for a user in their own
            domain
          * whether the persona can get a credential that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_ec2_list_credentials(self):
        """Test identity:list_credentials policy.

        This test must check:
          * whether the persona can list all credentials for themself
          * whether the persona can list credentials for a user in their own
            domain
          * whether the persona can list credentials for a user in another
            domain
        """
        pass

    @abc.abstractmethod
    def test_identity_ec2_delete_credential(self):
        """Test identity:ec2_delete_credential policy.

        This test must check
          * whether the persona can delete their own credential
          * whether the persona can delete a credential for a user in another
            domain
          * whether the persona can delete a credential for a user in their own
            domain
          * whether the persona can delete a credential that does not exist
        """
        pass


class SystemAdminTests(IdentityV3RbacEc2CredentialTest, base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_ec2_create_credential(self):
        # user can create their own credential
        user_id = self.persona.credentials.user_id
        resp = self.do_request(
            'create_user_ec2_credential',
            expected_status=201,
            user_id=user_id,
            **self.ec2_credential(project_id=self.test_project_1)
        )['credential']
        self.addCleanup(self.client.delete_user_ec2_credential,
                        user_id=user_id, access=resp['access'])
        # user can create credential for other user
        resp = self.do_request(
            'create_user_ec2_credential',
            expected_status=201,
            user_id=self.test_user_2,
            **self.ec2_credential(project_id=self.test_project_2)
        )['credential']
        self.addCleanup(self.client.delete_user_ec2_credential,
                        user_id=user_id, access=resp['access'])

    def test_identity_ec2_get_credential(self):
        # user can get their own credential
        user_id = self.persona.credentials.user_id
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=user_id,
            **self.ec2_credential(project_id=self.test_project_1)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=user_id, access=cred['access'])
        self.do_request('show_user_ec2_credential',
                        user_id=user_id, access=cred['access'])
        # user can get credential for other user
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=self.test_user_2,
            **self.ec2_credential(project_id=self.test_project_2)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=self.test_user_2, access=cred['access'])
        self.do_request('show_user_ec2_credential',
                        user_id=self.test_user_2, access=cred['access'])
        # non-existent credential is Not Found
        self.do_request(
            'show_user_ec2_credential',
            expected_status=exceptions.NotFound,
            user_id=self.test_user_2,
            access=data_utils.rand_uuid_hex())

    def test_identity_ec2_list_credentials(self):
        # user can list their own credentials
        user_id = self.persona.credentials.user_id
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=user_id,
            **self.ec2_credential(project_id=self.test_project_1)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=user_id, access=cred['access'])
        resp = self.do_request('list_user_ec2_credentials',
                               user_id=user_id)['credentials']
        self.assertIn(cred['access'], [c['access'] for c in resp])
        # user can list credentials for other user
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=self.test_user_2,
            **self.ec2_credential(project_id=self.test_project_2)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=self.test_user_2, access=cred['access'])
        resp = self.do_request('list_user_ec2_credentials',
                               user_id=self.test_user_2)['credentials']
        self.assertIn(cred['access'], [c['access'] for c in resp])

    def test_identity_ec2_delete_credential(self):
        # user can delete their own credential
        user_id = self.persona.credentials.user_id
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=user_id,
            **self.ec2_credential(project_id=self.test_project_1)
        )['credential']
        self.do_request(
            'delete_user_ec2_credential',
            expected_status=204,
            user_id=user_id, access=cred['access'])
        # user can delete another user's credential
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=self.test_user_2,
            **self.ec2_credential(project_id=self.test_project_2)
        )['credential']
        self.do_request(
            'delete_user_ec2_credential',
            expected_status=204,
            user_id=self.test_user_2, access=cred['access'])
        # non-existent credential is Not Found
        self.do_request(
            'delete_user_ec2_credential',
            expected_status=exceptions.NotFound,
            user_id=self.test_user_2,
            access=data_utils.rand_uuid_hex())


class SystemMemberTests(SystemAdminTests):

    credentials = ['system_member', 'system_admin']

    def test_identity_ec2_create_credential(self):
        # user can create their own credential
        user_id = self.persona.credentials.user_id
        resp = self.do_request(
            'create_user_ec2_credential',
            expected_status=201,
            user_id=user_id,
            **self.ec2_credential(project_id=self.test_project_1)
        )['credential']
        self.addCleanup(self.client.delete_user_ec2_credential,
                        user_id=user_id, access=resp['access'])
        # user cannot create credential for other user
        self.do_request(
            'create_user_ec2_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_2,
            **self.ec2_credential(project_id=self.test_project_2))

    def test_identity_ec2_delete_credential(self):
        # user can delete their own credential
        user_id = self.persona.credentials.user_id
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=user_id,
            **self.ec2_credential(project_id=self.test_project_1)
        )['credential']
        self.do_request(
            'delete_user_ec2_credential',
            expected_status=204,
            user_id=user_id, access=cred['access'])
        # user cannot delete another user's credential
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=self.test_user_2,
            **self.ec2_credential(project_id=self.test_project_2)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=self.test_user_2, access=cred['access'])
        self.do_request(
            'delete_user_ec2_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_2, access=cred['access'])
        # non-existent credential is Not Found
        self.do_request(
            'delete_user_ec2_credential',
            expected_status=exceptions.NotFound,
            user_id=self.test_user_2,
            access=data_utils.rand_uuid_hex())


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(IdentityV3RbacEc2CredentialTest, base.BaseIdentityTest):

    credentials = ['domain_admin', 'system_admin']

    def test_identity_ec2_create_credential(self):
        # user cannot create their own credential
        user_id = self.persona.credentials.user_id
        self.do_request(
            'create_user_ec2_credential',
            expected_status=exceptions.Forbidden,
            user_id=user_id,
            **self.ec2_credential(project_id=self.test_project_1))
        # user cannot create credential for user in own domain
        self.do_request(
            'create_user_ec2_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_1,
            **self.ec2_credential(project_id=self.test_project_1))
        # user cannot create credential for other user
        self.do_request(
            'create_user_ec2_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_2,
            **self.ec2_credential(project_id=self.test_project_2))

    def test_identity_ec2_get_credential(self):
        # user cannot get their own credential
        user_id = self.persona.credentials.user_id
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=user_id,
            **self.ec2_credential(project_id=self.test_project_1)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=user_id, access=cred['access'])
        self.do_request('show_user_ec2_credential',
                        expected_status=exceptions.Forbidden,
                        user_id=user_id, access=cred['access'])
        # user cannot get credential for user in own domain
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=self.test_user_1,
            **self.ec2_credential(project_id=self.test_project_1)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=self.test_user_1, access=cred['access'])
        self.do_request('show_user_ec2_credential',
                        expected_status=exceptions.Forbidden,
                        user_id=self.test_user_2, access=cred['access'])
        # user cannot get credential for other user
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=self.test_user_2,
            **self.ec2_credential(project_id=self.test_project_2)
            )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=self.test_user_2, access=cred['access'])
        self.do_request('show_user_ec2_credential',
                        expected_status=exceptions.Forbidden,
                        user_id=self.test_user_2, access=cred['access'])
        # non-existent credential is Not Found
        self.do_request(
            'show_user_ec2_credential',
            expected_status=exceptions.NotFound,
            user_id=self.test_user_2,
            access=data_utils.rand_uuid_hex())

    def test_identity_ec2_list_credentials(self):
        # user cannot list their own credentials
        user_id = self.persona.credentials.user_id
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=user_id,
            **self.ec2_credential(project_id=self.test_project_1)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=user_id, access=cred['access'])
        self.do_request('list_user_ec2_credentials',
                        expected_status=exceptions.Forbidden,
                        user_id=user_id)
        # user cannot list credentials for user in own domain
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=self.test_user_1,
            **self.ec2_credential(project_id=self.test_project_1)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=self.test_user_1, access=cred['access'])
        self.do_request('list_user_ec2_credentials',
                        expected_status=exceptions.Forbidden,
                        user_id=self.test_user_1)
        # user cannot list credentials for other user
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=self.test_user_2,
            **self.ec2_credential(project_id=self.test_project_2)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=self.test_user_2, access=cred['access'])
        self.do_request('list_user_ec2_credentials',
                        expected_status=exceptions.Forbidden,
                        user_id=self.test_user_2)

    def test_identity_ec2_delete_credential(self):
        # user cannot delete their own credential
        user_id = self.persona.credentials.user_id
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=user_id,
            **self.ec2_credential(project_id=self.test_project_1)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=user_id, access=cred['access'])
        self.do_request(
            'delete_user_ec2_credential',
            expected_status=exceptions.Forbidden,
            user_id=user_id, access=cred['access'])
        # user cannot delete credential for user in own domain
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=self.test_user_1,
            **self.ec2_credential(project_id=self.test_project_1)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=self.test_user_1, access=cred['access'])
        self.do_request(
            'delete_user_ec2_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_1, access=cred['access'])
        # user cannot delete another user's credential
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=self.test_user_2,
            **self.ec2_credential(project_id=self.test_project_2)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=self.test_user_2, access=cred['access'])
        self.do_request(
            'delete_user_ec2_credential',
            expected_status=exceptions.Forbidden,
            user_id=self.test_user_2, access=cred['access'])
        # non-existent credential is Not Found
        self.do_request(
            'delete_user_ec2_credential',
            expected_status=exceptions.NotFound,
            user_id=self.test_user_2,
            access=data_utils.rand_uuid_hex())


class DomainManagerTests(DomainAdminTests):

    credentials = ['domain_manager', 'system_admin']


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainAdminTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(SystemReaderTests):

    credentials = ['project_manager', 'system_admin']

    def test_identity_ec2_get_credential(self):
        # user can get their own credential
        user_id = self.persona.credentials.user_id
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=user_id,
            **self.ec2_credential(project_id=self.test_project_1)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=user_id, access=cred['access'])
        self.do_request('show_user_ec2_credential',
                        user_id=user_id, access=cred['access'])
        # user cannot get credential for other user
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=self.test_user_2,
            **self.ec2_credential(project_id=self.test_project_2)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=self.test_user_2, access=cred['access'])
        self.do_request('show_user_ec2_credential',
                        expected_status=exceptions.Forbidden,
                        user_id=self.test_user_2, access=cred['access'])
        # non-existent credential is Not Found
        self.do_request(
            'show_user_ec2_credential',
            expected_status=exceptions.NotFound,
            user_id=self.test_user_2,
            access=data_utils.rand_uuid_hex())

    def test_identity_ec2_list_credentials(self):
        # user can list their own credentials
        user_id = self.persona.credentials.user_id
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=user_id,
            **self.ec2_credential(project_id=self.test_project_1)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=user_id, access=cred['access'])
        resp = self.do_request('list_user_ec2_credentials',
                               user_id=user_id)['credentials']
        self.assertIn(cred['access'], [c['access'] for c in resp])
        # user cannot list credentials for other user
        cred = self.admin_credentials_client.create_user_ec2_credential(
            user_id=self.test_user_2,
            **self.ec2_credential(project_id=self.test_project_2)
        )['credential']
        self.addCleanup(
            self.admin_credentials_client.delete_user_ec2_credential,
            user_id=self.test_user_2, access=cred['access'])
        self.do_request('list_user_ec2_credentials',
                        expected_status=exceptions.Forbidden,
                        user_id=self.test_user_2)


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacRegionTests(rbac_base.IdentityV3RbacBaseTests,
                                metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacRegionTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.regions_client
        admin_client = cls.os_system_admin
        cls.admin_regions_client = admin_client.regions_client

    def region(self):
        return {'region_id': data_utils.rand_uuid_hex()}

    @abc.abstractmethod
    def test_identity_create_region(self):
        """Test identity:create_region policy.

        This test must check:
          * whether the persona can create a region
        """
        pass

    @abc.abstractmethod
    def test_identity_get_region(self):
        """Test identity:get_region policy.

        This test must check:
          * whether the persona can get a region
          * whether the persona can get a region that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_list_regions(self):
        """Test identity:list_regions policy.

        This test must check:
          * whether the persona can list all regions
        """
        pass

    @abc.abstractmethod
    def test_identity_update_region(self):
        """Test identity:update_region policy.

        This test must check:
          * whether the persona can update a region
          * whether the persona can update a region that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_region(self):
        """Test identity:delete_region policy.

        This test must check
          * whether the persona can delete a region
          * whether the persona can delete a region that does not exist
        """
        pass


class SystemAdminTests(IdentityV3RbacRegionTests, base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_create_region(self):
        region_id = self.do_request(
            'create_region', expected_status=201,
            **self.region())['region']['id']
        self.addCleanup(
            self.admin_regions_client.delete_region,
            region_id=region_id)

    def test_identity_get_region(self):
        region_id = self.admin_regions_client.create_region(
            **self.region())['region']['id']
        self.addCleanup(
            self.admin_regions_client.delete_region,
            region_id=region_id)
        self.do_request('show_region', region_id=region_id)
        # user gets a 404 for nonexistent region
        self.do_request('show_region', expected_status=exceptions.NotFound,
                        region_id=data_utils.rand_uuid_hex())

    def test_identity_list_regions(self):
        region_id = self.admin_regions_client.create_region(
            **self.region())['region']['id']
        self.addCleanup(
            self.admin_regions_client.delete_region,
            region_id=region_id)
        resp = self.do_request('list_regions')
        self.assertIn(region_id, [e['id'] for e in resp['regions']])

    def test_identity_update_region(self):
        region_id = self.admin_regions_client.create_region(
            **self.region())['region']['id']
        self.addCleanup(
            self.admin_regions_client.delete_region,
            region_id=region_id)
        self.do_request('update_region',
                        region_id=region_id,
                        description=data_utils.rand_uuid_hex())
        # user gets a 404 for nonexistent region
        self.do_request('update_region', expected_status=exceptions.NotFound,
                        region_id=data_utils.rand_uuid_hex(),
                        description=data_utils.rand_uuid_hex())

    def test_identity_delete_region(self):
        region_id = self.admin_regions_client.create_region(
            **self.region())['region']['id']
        self.do_request('delete_region', expected_status=204,
                        region_id=region_id)
        # user gets a 404 for nonexistent region
        self.do_request('delete_region', expected_status=exceptions.NotFound,
                        region_id=data_utils.rand_uuid_hex())


class SystemMemberTests(SystemAdminTests, base.BaseIdentityTest):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_region(self):
        self.do_request(
            'create_region', expected_status=exceptions.Forbidden,
            **self.region())

    def test_identity_update_region(self):
        region_id = self.admin_regions_client.create_region(
            **self.region())['region']['id']
        self.addCleanup(
            self.admin_regions_client.delete_region,
            region_id=region_id)
        self.do_request('update_region', expected_status=exceptions.Forbidden,
                        region_id=region_id,
                        description=data_utils.rand_uuid_hex())
        # user gets a 403 for nonexistent region
        self.do_request('update_region', expected_status=exceptions.Forbidden,
                        region_id=data_utils.rand_uuid_hex(),
                        description=data_utils.rand_uuid_hex())

    def test_identity_delete_region(self):
        region_id = self.admin_regions_client.create_region(
            **self.region())['region']['id']
        self.do_request('delete_region',
                        expected_status=exceptions.Forbidden,
                        region_id=region_id)
        # user gets a 403 for nonexistent region
        self.do_request('delete_region', expected_status=exceptions.Forbidden,
                        region_id=data_utils.rand_uuid_hex())


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemReaderTests, base.BaseIdentityTest):

    credentials = ['domain_admin', 'system_admin']


class DomainManagerTests(DomainAdminTests, base.BaseIdentityTest):

    credentials = ['domain_manager', 'system_admin']


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainReaderTests):

    credentials = ['project_manager', 'system_admin']


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from tempest.lib.common import rest_client

from keystone_tempest_plugin.services.identity import clients


class IdentityProvidersClient(clients.Federation):

    subpath_suffix = 'identity_providers'

    def create_identity_provider(self, idp_id, **kwargs):
        """Create an identity provider.

        :param str idp_id: The ID to be used to create the Identity Provider.
        :param kwargs: All optional attributes: description (str), enabled
                       (boolean) and remote_ids (list).
        """
        put_body = json.dumps({'identity_provider': kwargs})
        return self._put(idp_id, put_body)

    def list_identity_providers(self):
        """List the identity providers."""
        return self._get()

    def show_identity_provider(self, idp_id):
        """Get an identity provider."""
        return self._get(idp_id)

    def delete_identity_provider(self, idp_id):
        """Delete an identity provider."""
        return self._delete(idp_id)

    def update_identity_provider(self, idp_id, **kwargs):
        """Update an identity provider.

        :param str idp_id: The ID from the Identity Provider to be updated.
        :param kwargs: All optional attributes to update: description (str),
                       enabled (boolean) and remote_ids (list).
        """
        patch_body = json.dumps({'identity_provider': kwargs})
        return self._patch(idp_id, patch_body)

    def add_protocol_and_mapping(self, idp_id, protocol_id, mapping_id):
        """Add a protocol and mapping to an identity provider."""
        put_body = json.dumps({'protocol': {'mapping_id': mapping_id}})
        url = '%s/%s/%s' % (
            self._build_path(entity_id=idp_id), 'protocols', protocol_id)
        resp, body = self.put(url, put_body)
        self.expected_success(201, resp.status)
        body = json.loads(body.decode('utf-8'))
        return rest_client.ResponseBody(resp, body)

    def delete_protocol_and_mapping(self, idp_id, protocol_id):
        """Delete a protocol and mapping from an identity provider."""
        url = '%s/%s/%s' % (
            self._build_path(entity_id=idp_id), 'protocols', protocol_id)
        resp, body = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def get_protocol_and_mapping(self, idp_id, protocol_id):
        """Get a protocol and mapping from an identity provider."""
        url = '%s/%s/%s' % (
            self._build_path(entity_id=idp_id), 'protocols', protocol_id)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body.decode('utf-8'))
        return rest_client.ResponseBody(resp, body)

    def list_protocols_and_mappings(self, idp_id):
        """List the protocols and mappings from an identity provider."""
        url = '%s/%s' % (self._build_path(entity_id=idp_id), 'protocols')
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body.decode('utf-8'))
        return rest_client.ResponseBody(resp, body)

    def update_protocol_mapping(self, idp_id, protocol_id, mapping_id):
        """Update the identity provider protocol with a new mapping."""
        patch_body = json.dumps({'protocol': {'mapping_id': mapping_id}})
        url = '%s/%s/%s' % (
            self._build_path(entity_id=idp_id), 'protocols', protocol_id)
        resp, body = self.patch(url, patch_body)
        self.expected_success(200, resp.status)
        body = json.loads(body.decode('utf-8'))
        return rest_client.ResponseBody(resp, body)



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from keystone_tempest_plugin.tests.api.identity.v3 import fixtures
from keystone_tempest_plugin.tests import base


class MappingRulesTest(base.BaseIdentityTest):

    _MAPPING_REF = fixtures.mapping_ref()

    def _assert_mapping_rules_attributes(self, mapping, mapping_id,
                                         mapping_ref=None):
        self.assertIn('id', mapping)
        self.assertEqual(mapping_id, mapping['id'])

        self.assertIn('rules', mapping)

        if mapping_ref:
            self.assertCountEqual(mapping_ref['rules'], mapping['rules'])

    def _create_mapping_rule(self, mapping_id, mapping_ref):
        mapping = self.mappings_client.create_mapping_rule(
            mapping_id, mapping_ref)['mapping']
        self.addCleanup(self.mappings_client.delete_mapping_rule, mapping_id)
        return mapping

    @decorators.idempotent_id('4ca48c01-b6da-4759-acb6-007e15ad712a')
    def test_mapping_rules_create(self):
        mapping_id = data_utils.rand_uuid_hex()
        mapping = self._create_mapping_rule(mapping_id, self._MAPPING_REF)
        self._assert_mapping_rules_attributes(
            mapping, mapping_id, self._MAPPING_REF)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('341dac45-ce1f-4f15-afdc-1f9a7d7d7c40')
    def test_mapping_rules_create_without_mandatory_attributes_fails(self):
        mapping_id = data_utils.rand_uuid_hex()
        self.assertRaises(
            lib_exc.BadRequest,
            self.mappings_client.create_mapping_rule,
            mapping_id,
            {})

    @decorators.idempotent_id('8db213e3-1db0-48c6-863c-7a3ed23577ec')
    def test_mapping_rules_get(self):
        mapping_id = data_utils.rand_uuid_hex()
        mapping_create = self._create_mapping_rule(
            mapping_id, self._MAPPING_REF)

        mapping_get = self.mappings_client.show_mapping_rule(mapping_id)[
            'mapping']
        self._assert_mapping_rules_attributes(
            mapping_get, mapping_id, mapping_create)

    @decorators.idempotent_id('bb80b242-2a6a-4d29-b45f-4035be574a6e')
    def test_mapping_rules_list(self):
        mapping_ids = []
        for _ in range(3):
            mapping_id = data_utils.rand_uuid_hex()
            self._create_mapping_rule(mapping_id, self._MAPPING_REF)
            mapping_ids.append(mapping_id)

        mappings_list = self.mappings_client.list_mapping_rules()['mappings']
        fetched_ids = [mapping['id'] for mapping in mappings_list]

        for mapping_id in mapping_ids:
            self.assertIn(mapping_id, fetched_ids)

    @decorators.idempotent_id('1fc5d104-faf5-4809-8c89-29b5c1666a96')
    def test_mapping_rule_update(self):
        mapping_id = data_utils.rand_uuid_hex()
        mapping_ref = fixtures.mapping_ref()
        mapping = self._create_mapping_rule(mapping_id, mapping_ref)

        new_local = [{'group': {'id': data_utils.rand_uuid_hex()}}]
        mapping_ref['rules'][0]['local'] = new_local

        mapping = self.mappings_client.update_mapping_rule(
            mapping_id, mapping_ref)['mapping']
        self._assert_mapping_rules_attributes(
            mapping, mapping_id, mapping_ref)

        mapping_get = self.mappings_client.show_mapping_rule(mapping_id)[
            'mapping']
        self._assert_mapping_rules_attributes(
            mapping_get, mapping_id, mapping)



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from keystone_tempest_plugin.tests.api.identity.v3 import fixtures
from keystone_tempest_plugin.tests import base


class IndentityProvidersTest(base.BaseIdentityTest):

    def _assert_identity_provider_attributes(self, idp, idp_id, idp_ref=None):
        self.assertIn('id', idp)
        self.assertEqual(idp_id, idp['id'])

        # Check the optional attributes have been set
        self.assertIn('description', idp)
        self.assertIn('enabled', idp)
        self.assertIn('remote_ids', idp)

        if idp_ref:
            self.assertEqual(idp_ref['description'], idp['description'])

            if 'enabled' in idp_ref:
                self.assertEqual(idp_ref['enabled'], idp['enabled'])

            if 'remote_ids' in idp_ref:
                self.assertCountEqual(idp_ref['remote_ids'], idp['remote_ids'])

    def _create_idp(self, idp_id, idp_ref):
        idp = self.idps_client.create_identity_provider(
            idp_id, **idp_ref)['identity_provider']
        if not idp_ref.get('domain_id'):
            self.addCleanup(
                self.keystone_manager.domains_client.delete_domain,
                idp['domain_id'])
            self.addCleanup(
                self.keystone_manager.domains_client.update_domain,
                idp['domain_id'], enabled=False)
        self.addCleanup(
            self.idps_client.delete_identity_provider, idp_id)
        return idp

    @decorators.idempotent_id('09450910-b816-4150-8513-a2fd4628a0c3')
    def test_identity_provider_create(self):
        idp_id = data_utils.rand_uuid_hex()
        idp_ref = fixtures.idp_ref()
        idp = self._create_idp(idp_id, idp_ref)

        # The identity provider is disabled by default
        idp_ref['enabled'] = False

        # The remote_ids attribute should be set to an empty list by default
        idp_ref['remote_ids'] = []

        self._assert_identity_provider_attributes(idp, idp_id, idp_ref)

    @decorators.idempotent_id('f430a337-545d-455e-bb6c-cb0fdf4be5c1')
    def test_identity_provider_create_with_enabled_true(self):
        idp_id = data_utils.rand_uuid_hex()
        idp_ref = fixtures.idp_ref(enabled=True)
        idp = self._create_idp(idp_id, idp_ref)

        self._assert_identity_provider_attributes(idp, idp_id, idp_ref)

    @decorators.idempotent_id('238e6163-d600-4f59-9982-c621f057221d')
    def test_identity_provider_create_with_remote_ids(self):
        idp_id = data_utils.rand_uuid_hex()
        remote_ids = [data_utils.rand_uuid_hex(), data_utils.rand_uuid_hex()]
        idp_ref = fixtures.idp_ref(remote_ids=remote_ids)
        idp = self._create_idp(idp_id, idp_ref)

        self._assert_identity_provider_attributes(idp, idp_id, idp_ref)

    @decorators.idempotent_id('8a7817ad-27f8-436b-9cbe-46aa20989beb')
    def test_identity_provider_get(self):
        idp_id = data_utils.rand_uuid_hex()
        idp_create = self._create_idp(idp_id, fixtures.idp_ref())

        idp_get = self.idps_client.show_identity_provider(
            idp_id)['identity_provider']
        self._assert_identity_provider_attributes(idp_get, idp_id, idp_create)

    @decorators.idempotent_id('cbfe5de9-c58a-4810-950c-2acdf985879d')
    def test_identity_provider_list(self):
        idp_ids = []
        for _ in range(3):
            idp_id = data_utils.rand_uuid_hex()
            self._create_idp(idp_id, fixtures.idp_ref())
            idp_ids.append(idp_id)

        idp_list = self.idps_client.list_identity_providers()[
            'identity_providers']
        fetched_ids = [fetched_idp['id'] for fetched_idp in idp_list]

        for idp_id in idp_ids:
            self.assertIn(idp_id, fetched_ids)

    @decorators.idempotent_id('36a0d9f0-9517-4139-85d0-f78d905aece5')
    def test_identity_provider_update(self):
        idp_id = data_utils.rand_uuid_hex()
        idp = self._create_idp(idp_id, fixtures.idp_ref(enabled=True))

        # The identity provider should be enabled
        self.assertTrue(idp['enabled'])

        idp = self.idps_client.update_identity_provider(
            idp_id, enabled=False)['identity_provider']

        # The identity provider should be disabled
        self.assertFalse(idp['enabled'])

        idp_get = self.idps_client.show_identity_provider(
            idp_id)['identity_provider']
        self.assertFalse(idp_get['enabled'])

    def _assert_protocol_attributes(self, protocol, protocol_id,
                                    mapping_id=None):
        self.assertIn('id', protocol)
        self.assertEqual(protocol_id, protocol['id'])

        self.assertIn('mapping_id', protocol)
        if mapping_id:
            self.assertEqual(mapping_id, protocol['mapping_id'])

    def _create_identity_provider_and_mapping(self):
        # Create an identity provider
        idp_id = data_utils.rand_uuid_hex()
        self._create_idp(idp_id, fixtures.idp_ref(enabled=True))

        # Create a mapping rule
        mapping_id = data_utils.rand_uuid_hex()
        self.mappings_client.create_mapping_rule(
            mapping_id, fixtures.mapping_ref())
        self.addCleanup(self.mappings_client.delete_mapping_rule, mapping_id)

        return idp_id, mapping_id

    def _create_protocol(self, idp_id, protocol_id, mapping_id):
        protocol = self.idps_client.add_protocol_and_mapping(
            idp_id, protocol_id, mapping_id)['protocol']
        self.addCleanup(
            self.idps_client.delete_protocol_and_mapping, idp_id, protocol_id)
        return protocol

    @decorators.idempotent_id('f5bdf482-1ad3-4aad-b52e-8fe7c1361104')
    def test_add_protocol_to_identity_provider(self):
        idp_id, mapping_id = self._create_identity_provider_and_mapping()

        # Now we try to add a protocol to the identity provider
        protocol_id = data_utils.rand_uuid_hex()
        protocol = self._create_protocol(idp_id, protocol_id, mapping_id)

        self._assert_protocol_attributes(protocol, protocol_id, mapping_id)

    @decorators.idempotent_id('613d073b-0db7-41aa-b4df-5a1848cde0b3')
    def test_get_protocol_from_identity_provider(self):
        idp_id, mapping_id = self._create_identity_provider_and_mapping()

        # Add a protocol to the identity provider
        protocol_id = data_utils.rand_uuid_hex()
        self._create_protocol(idp_id, protocol_id, mapping_id)

        # Try to get the protocol
        protocol = self.idps_client.get_protocol_and_mapping(
            idp_id, protocol_id)['protocol']
        self._assert_protocol_attributes(protocol, protocol_id, mapping_id)

    @decorators.idempotent_id('6e6501ac-edae-4dc2-bb68-7a2b58602383')
    def test_list_protocols_from_identity_provider(self):
        idp_id, mapping_id = self._create_identity_provider_and_mapping()

        protocol_ids = []
        for _ in range(3):
            protocol_id = data_utils.rand_uuid_hex()
            self._create_protocol(idp_id, protocol_id, mapping_id)
            protocol_ids.append(protocol_id)

        protocols_list = self.idps_client.list_protocols_and_mappings(idp_id)[
            'protocols']
        fetched_ids = [fetched['id'] for fetched in protocols_list]

        for protocol_id in protocol_ids:
            self.assertIn(protocol_id, fetched_ids)

    @decorators.idempotent_id('680d36df-d6b9-4695-be29-6fc2065c5dde')
    def test_update_mapping_from_identity_provider_protocol(self):
        idp_id, mapping_id = self._create_identity_provider_and_mapping()

        # Add a protocol to the identity provider
        protocol_id = data_utils.rand_uuid_hex()
        protocol = self._create_protocol(idp_id, protocol_id, mapping_id)

        # Create another mapping
        new_mapping_id = data_utils.rand_uuid_hex()
        self.mappings_client.create_mapping_rule(
            new_mapping_id, fixtures.mapping_ref())

        # Update the identity provider protocol
        protocol = self.idps_client.update_protocol_mapping(
            idp_id, protocol_id, new_mapping_id)['protocol']

        self._assert_protocol_attributes(protocol, protocol_id, new_mapping_id)

    @decorators.idempotent_id('04fdf262-af91-4a68-a1cf-794c6d2f2eeb')
    def test_add_protocol_to_identity_provider_unknown_mapping(self):
        # Create an identity provider
        idp_id = data_utils.rand_uuid_hex()
        self._create_idp(idp_id, fixtures.idp_ref(enabled=True))

        # Now we try to add a protocol to the identity provider using
        # a non existent mapping ID
        mapping_id = data_utils.rand_uuid_hex()
        protocol_id = data_utils.rand_uuid_hex()
        self.assertRaises(
            lib_exc.BadRequest,
            self._create_protocol,
            idp_id,
            protocol_id,
            mapping_id)

    @decorators.idempotent_id('c73311e7-c207-4c11-998f-532a91f1b0d1')
    def test_update_protocol_from_identity_provider_unknown_mapping(self):
        idp_id, mapping_id = self._create_identity_provider_and_mapping()

        # Add a protocol to the identity provider
        protocol_id = data_utils.rand_uuid_hex()
        self._create_protocol(idp_id, protocol_id, mapping_id)

        # Update the identity provider protocol using a non existent
        # mapping_id
        new_mapping_id = data_utils.rand_uuid_hex()
        self.assertRaises(
            lib_exc.BadRequest,
            self.idps_client.update_protocol_mapping,
            idp_id,
            protocol_id,
            new_mapping_id)



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacGroupTest(rbac_base.IdentityV3RbacBaseTests,
                              metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacGroupTest, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.groups_client
        cls.users_client = cls.persona.users_v3_client
        cls.admin_client = cls.os_system_admin
        cls.admin_groups_client = cls.admin_client.groups_client
        cls.admin_users_client = cls.admin_client.users_v3_client
        cls.admin_domains_client = cls.admin_client.domains_client

    def setUp(self):
        super(IdentityV3RbacGroupTest, self).setUp()
        self.own_domain = self.persona.credentials.domain_id
        self.other_domain = self.admin_domains_client.create_domain(
            name=data_utils.rand_name())['domain']['id']
        self.addCleanup(self.admin_domains_client.delete_domain,
                        self.other_domain)
        self.addCleanup(self.admin_domains_client.update_domain,
                        domain_id=self.other_domain, enabled=False)

    def group(self, domain_id=None):
        group = {}
        name = data_utils.rand_name('group')
        group['name'] = name
        if domain_id:
            group['domain_id'] = domain_id
        return group

    @abc.abstractmethod
    def test_identity_create_group(self):
        """Test identity:create_group policy.

        This test must check:
          * whether the persona can create an arbitrary group
          * whether the persona can create a group in another domain
          * whether the persona can create a group in their own domain
        """
        pass

    @abc.abstractmethod
    def test_identity_get_group(self):
        """Test identity:get_group policy.

        This test must check:
          * whether the persona can get an arbitrary group
          * whether the persona can get a group in another domain
          * whether the persona can get a group in their own domain
          * whether the persona can get a group that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_list_groups(self):
        """Test identity:list_groups policy.

        This test must check:
          * whether the persona can list all groups
          * whether the result list is appropriately filtered to domain
        """
        pass

    @abc.abstractmethod
    def test_identity_list_groups_for_user(self):
        """Test identity:list_groups_for_user policy.

        This test must check:
          * whether the persona can list groups for an arbitrary user
          * whether the persona can see groups in their own domain for user in
            their own domain
          * whether the persona can see groups in another domain for user in
            their own domain
          * whether the persona can see groups in their own domain for user in
            another domain
          * whether the persona can see groups in another domain for user in
            another domain
          * whether the persona can list groups for a nonexistent user
        """
        pass

    @abc.abstractmethod
    def test_identity_update_group(self):
        """Test identity:update_groups policy.

        This test must check:
          * whether the persona can update an arbitrary group
          * whether the persona can update a group in another domain
          * whether the persona can update a group in their own domain
          * whether the persona can update a group that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_group(self):
        """Test identity:delete_group policy.

        This test must check
          * whether the persona can delete an arbitrary group
          * whether the persona can delete a group in another domain
          * whether the persona can delete a group in their own domain
          * whether the persona can delete a group that does not exist
        """
        pass

    @abc.abstractmethod
    def test_identity_list_users_in_group(self):
        """Test identity:list_users_in_group policy.

        This test must check
          * whether the persona can list users in an arbitrary group
          * whether the persona can see users in their own domain for group in
            their own domain
          * whether the persona can see users in another domain for group in
            their own domain
          * whether the persona can see users in their own domain for group in
            another domain
          * whether the persona can see users in another domain for group in
            another domain
          * whether the persona can list users for a nonexistent group
        """
        pass

    @abc.abstractmethod
    def test_identity_add_user_to_group(self):
        """Test identity:add_user_to_group policy.

        This test must check
          * whether the persona can add an arbitrary user to an arbitrary group
          * whether the persona can add a user in their own domain to a group
            in their own domain
          * whether the persona can add a user in another domain to a group in
            their own domain
          * whether the persona can add a user in their own domain to a group
            in another domain
          * whether the persona can add a user in another domain to a group in
            their own domain
          * whether the persona can add a nonexistent user to a group
          * whether the persona can add a user to a nonexistent group
        """
        pass

    @abc.abstractmethod
    def test_identity_remove_user_from_group(self):
        """Test identity:remove_user_from_group policy.

        This test must check
          * whether the persona can remove an arbitrary user from an arbitrary
            group
          * whether the persona can remove a user in their own domain from a
            group in their own domain
          * whether the persona can remove a user in another domain from a
            group in their own domain
          * whether the persona can remove a user in their own domain from a
            group in another domain
          * whether the persona can remove a user in another domain from a
            group in their own domain
          * whether the persona can remove a nonexistent user from a group
          * whether the persona can remove a user from a nonexistent group
        """
        pass

    @abc.abstractmethod
    def test_identity_check_user_in_group(self):
        """Test identity:check_user_in_group policy.

        This test must check
          * whether the persona can check if an arbitrary user is in an
            arbitrary group
          * whether the persona can check if a user in their own domain is in a
            group in their own domain
          * whether the persona can check if a user in another domain is in a
            group in their own domain
          * whether the persona can check if a user in their own domain is in a
            group in another domain
          * whether the persona can check if a user in another domain is in a
            group in another domain
          * whether the persona can check if a nonexistent user is in a group
          * whether the persona can check if a user is in a nonexistent group
        """
        pass


class SystemAdminTests(IdentityV3RbacGroupTest, base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_create_group(self):
        resp = self.do_request('create_group', expected_status=201,
                               **self.group())
        self.addCleanup(self.admin_groups_client.delete_group,
                        resp['group']['id'])

    def test_identity_get_group(self):
        group = self.admin_groups_client.create_group(**self.group())['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        # user can get an arbitrary group
        self.do_request('show_group', group_id=group['id'])
        # user gets a 404 for nonexistent group
        self.do_request('show_group', expected_status=exceptions.NotFound,
                        group_id='fakegroup')

    def test_identity_list_groups(self):
        group = self.admin_groups_client.create_group(**self.group())['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        resp = self.do_request('list_groups')
        self.assertIn(group['id'], set(g['id'] for g in resp['groups']))

    def test_identity_list_groups_for_user(self):
        group = self.admin_groups_client.create_group(**self.group())['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        user = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'))['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user['id'])
        self.admin_groups_client.add_group_user(group['id'], user['id'])
        resp = self.do_request('list_user_groups', client=self.users_client,
                               user_id=user['id'])
        self.assertIn(group['id'], set(g['id'] for g in resp['groups']))
        self.do_request('list_user_groups', client=self.users_client,
                        expected_status=exceptions.NotFound,
                        user_id='fakeuser')

    def test_identity_update_group(self):
        group = self.admin_groups_client.create_group(**self.group())['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        # user can update an arbitrary group
        group_update = {
            'group_id': group['id'],
            'description': data_utils.arbitrary_string
        }
        self.do_request('update_group', **group_update)
        # user gets a 404 for nonexistent group
        group_update = {
            'group_id': 'fakegroup',
            'description': data_utils.arbitrary_string
        }
        self.do_request('update_group', expected_status=exceptions.NotFound,
                        **group_update)

    def test_identity_delete_group(self):
        group = self.admin_groups_client.create_group(**self.group())['group']
        # user can delete an arbitrary group
        self.do_request('delete_group', expected_status=204,
                        group_id=group['id'])
        # user gets a 404 for nonexistent group
        self.do_request('delete_group', expected_status=exceptions.NotFound,
                        group_id='fakegroup')

    def test_identity_list_users_in_group(self):
        group = self.admin_groups_client.create_group(**self.group())['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        user = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'))['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user['id'])
        self.admin_groups_client.add_group_user(group['id'], user['id'])
        resp = self.do_request('list_group_users', group_id=group['id'])
        user_ids = set(u['id'] for u in resp['users'])
        self.assertEqual(1, len(user_ids))
        self.assertIn(user['id'], user_ids)

    def test_identity_add_user_to_group(self):
        group = self.admin_groups_client.create_group(**self.group())['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        user = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'))['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user['id'])
        # user can add a user to a group
        self.do_request('add_group_user', expected_status=204,
                        group_id=group['id'], user_id=user['id'])
        # user gets a 404 for nonexistent group
        self.do_request('add_group_user', expected_status=exceptions.NotFound,
                        group_id='fakegroup', user_id=user['id'])
        # user gets a 404 for nonexistent user
        self.do_request('add_group_user', expected_status=exceptions.NotFound,
                        group_id=group['id'], user_id='fakeuser')

    def test_identity_remove_user_from_group(self):
        group = self.admin_groups_client.create_group(**self.group())['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        user = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'))['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user['id'])
        self.admin_groups_client.add_group_user(group['id'], user['id'])
        # user can remove a user from a group
        self.do_request('delete_group_user', expected_status=204,
                        group_id=group['id'], user_id=user['id'])
        # user gets a 404 for nonexistent group
        self.do_request('delete_group_user',
                        expected_status=exceptions.NotFound,
                        group_id='fakegroup', user_id=user['id'])
        # user gets a 404 for nonexistent user
        self.do_request('delete_group_user',
                        expected_status=exceptions.NotFound,
                        group_id=group['id'], user_id='fakeuser')

    def test_identity_check_user_in_group(self):
        group = self.admin_groups_client.create_group(**self.group())['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        user = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'))['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user['id'])
        self.admin_groups_client.add_group_user(group['id'], user['id'])
        # user can check if a user is in a group
        self.do_request('check_group_user_existence', expected_status=204,
                        group_id=group['id'], user_id=user['id'])
        # user gets a 404 for nonexistent group
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.NotFound,
                        group_id='fakegroup', user_id=user['id'])
        # user gets a 404 for nonexistent user
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.NotFound,
                        group_id=group['id'], user_id='fakeuser')


class SystemMemberTests(SystemAdminTests):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_group(self):
        self.do_request('create_group', expected_status=exceptions.Forbidden,
                        **self.group())

    def test_identity_update_group(self):
        group = self.admin_groups_client.create_group(**self.group())['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        # user cannot update an arbitrary group
        group_update = {
            'group_id': group['id'],
            'description': data_utils.arbitrary_string
        }
        self.do_request('update_group', expected_status=exceptions.Forbidden,
                        **group_update)
        # user gets a 403 for nonexistent group
        group_update = {
            'group_id': 'fakegroup',
            'description': data_utils.arbitrary_string
        }
        self.do_request('update_group', expected_status=exceptions.Forbidden,
                        **group_update)

    def test_identity_delete_group(self):
        group = self.admin_groups_client.create_group(**self.group())['group']
        # user cannot delete an arbitrary group
        self.do_request('delete_group', expected_status=exceptions.Forbidden,
                        group_id=group['id'])
        # user gets a 403 for nonexistent group
        self.do_request('delete_group', expected_status=exceptions.Forbidden,
                        group_id=group['id'])

    def test_identity_add_user_to_group(self):
        group = self.admin_groups_client.create_group(**self.group())['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        user = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'))['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user['id'])
        # user cannot add a user to a group
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group['id'], user_id=user['id'])
        # user gets a 403 for nonexistent group
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id='fakegroup', user_id=user['id'])
        # user gets a 403 for nonexistent user
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group['id'], user_id='fakeuser')

    def test_identity_remove_user_from_group(self):
        group = self.admin_groups_client.create_group(**self.group())['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        user = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'))['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user['id'])
        self.admin_groups_client.add_group_user(group['id'], user['id'])
        # user cannot remove a user from a group
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group['id'], user_id=user['id'])
        # user gets a 403 for nonexistent group
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id='fakegroup', user_id=user['id'])
        # user gets a 403 for nonexistent user
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group['id'], user_id='fakeuser')


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemAdminTests):

    credentials = ['domain_admin', 'system_admin']

    def test_identity_list_groups(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        resp = self.do_request('list_groups')
        # user can get groups in own domain
        self.assertIn(group1['id'], set(g['id'] for g in resp['groups']))
        # user cannot get groups in other domain
        self.assertNotIn(group2['id'], set(g['id'] for g in resp['groups']))

    def test_identity_list_groups_for_user(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.other_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        self.admin_groups_client.add_group_user(group1['id'], user1['id'])
        self.admin_groups_client.add_group_user(group1['id'], user2['id'])
        self.admin_groups_client.add_group_user(group2['id'], user1['id'])
        self.admin_groups_client.add_group_user(group2['id'], user2['id'])
        resp = self.do_request('list_user_groups', client=self.users_client,
                               user_id=user1['id'])
        # user can list groups in own domain for user in own domain
        self.assertIn(group1['id'], set(g['id'] for g in resp['groups']))
        # user cannot list groups in other domain for user in own domain
        self.assertNotIn(group2['id'], set(g['id'] for g in resp['groups']))
        # user cannot list groups for user in other domain
        resp = self.do_request('list_user_groups', client=self.users_client,
                               user_id=user2['id'])
        self.assertNotIn(group2['id'], set(g['id'] for g in resp['groups']))
        # user gets a 404 for nonexistent user
        self.do_request('list_user_groups', client=self.users_client,
                        expected_status=exceptions.NotFound,
                        user_id='fakeuser')

    def test_identity_list_users_in_group(self):
        group = self.admin_groups_client.create_group(**self.group())['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        user = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'))['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user['id'])
        self.admin_groups_client.add_group_user(group['id'], user['id'])
        resp = self.do_request('list_group_users', group_id=group['id'])
        user_ids = set(u['id'] for u in resp['users'])
        # request is allowed but the user will get filtered out on domain
        # scoped request
        self.assertEqual(0, len(user_ids))


class DomainManagerTests(DomainAdminTests):

    credentials = ['domain_manager', 'system_admin']

    def test_identity_create_group(self):
        # user can create group in own domain
        resp = self.do_request('create_group',
                               expected_status=201,
                               **self.group(domain_id=self.own_domain))
        self.addCleanup(self.admin_groups_client.delete_group,
                        resp['group']['id'])
        # user cannot create group in another domain
        self.do_request('create_group',
                        expected_status=exceptions.Forbidden,
                        **self.group(domain_id=self.other_domain))

    def test_identity_get_group(self):
        group = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        # user can get group in own domain
        self.do_request('show_group', group_id=group['id'])
        # user cannot get group in other domain
        group = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        self.do_request('show_group', expected_status=exceptions.Forbidden,
                        group_id=group['id'])
        # user gets a 403 for nonexistent group
        self.do_request('show_group', expected_status=exceptions.Forbidden,
                        group_id='fakegroup')

    def test_identity_list_groups_for_user(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.other_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        self.admin_groups_client.add_group_user(group1['id'], user1['id'])
        self.admin_groups_client.add_group_user(group1['id'], user2['id'])
        self.admin_groups_client.add_group_user(group2['id'], user1['id'])
        self.admin_groups_client.add_group_user(group2['id'], user2['id'])
        resp = self.do_request('list_user_groups', client=self.users_client,
                               user_id=user1['id'])
        # user can list groups in own domain for user in own domain
        self.assertIn(group1['id'], set(g['id'] for g in resp['groups']))
        # user cannot list groups in other domain for user in own domain
        self.assertNotIn(group2['id'], set(g['id'] for g in resp['groups']))
        # user cannot list groups for user in other domain
        resp = self.do_request('list_user_groups', client=self.users_client,
                               expected_status=exceptions.Forbidden,
                               user_id=user2['id'])
        # user gets a 403 for nonexistent user
        self.do_request('list_user_groups', client=self.users_client,
                        expected_status=exceptions.Forbidden,
                        user_id='fakeuser')

    def test_identity_update_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        # user can update a group in own domain
        group_update = {
            'group_id': group1['id'],
            'description': data_utils.arbitrary_string
        }
        self.do_request('update_group', **group_update)
        # user cannot update a group in other domain
        group_update = {
            'group_id': group2['id'],
            'description': data_utils.arbitrary_string
        }
        self.do_request('update_group', expected_status=exceptions.Forbidden,
                        **group_update)
        # user gets a 403 for nonexistent group
        group_update = {
            'group_id': 'fakegroup',
            'description': data_utils.arbitrary_string
        }
        self.do_request('update_group', expected_status=exceptions.Forbidden,
                        **group_update)

    def test_identity_delete_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        # user can delete a group in own domain
        self.do_request('delete_group', expected_status=204,
                        group_id=group1['id'])
        # user cannot delete a group in other domain
        self.do_request('delete_group', expected_status=exceptions.Forbidden,
                        group_id=group2['id'])
        # user gets a 404 for nonexistent group
        self.do_request('delete_group', expected_status=exceptions.NotFound,
                        group_id='fakegroup')

    def test_identity_list_users_in_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.other_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        self.admin_groups_client.add_group_user(group1['id'], user1['id'])
        self.admin_groups_client.add_group_user(group1['id'], user2['id'])
        self.admin_groups_client.add_group_user(group2['id'], user1['id'])
        self.admin_groups_client.add_group_user(group2['id'], user2['id'])
        resp = self.do_request('list_group_users', group_id=group1['id'])
        # user can list users in own domain for group in own domain
        self.assertIn(user1['id'], set(u['id'] for u in resp['users']))
        # user cannot list users in another domain for group in own domain
        self.assertNotIn(user2['id'], set(u['id'] for u in resp['users']))
        # user cannot list users for group in another domain
        self.do_request('list_group_users',
                        expected_status=exceptions.Forbidden,
                        group_id=group2['id'])

    def test_identity_add_user_to_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.other_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        # user can add a user in own domain to a group in own domain
        self.do_request('add_group_user', expected_status=204,
                        group_id=group1['id'], user_id=user1['id'])
        # user cannot add a user in another domain to a group in own domain
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id=user2['id'])
        # user cannot add a user in own domain to a group in another domain
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user1['id'])
        # user cannot add a user in another domain to a group in another domain
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user2['id'])
        # user gets a 403 for nonexistent group
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id='fakegroup', user_id=user1['id'])
        # user gets a 403 for nonexistent user
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id='fakeuser')

    def test_identity_remove_user_from_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.other_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        self.admin_groups_client.add_group_user(group1['id'], user1['id'])
        self.admin_groups_client.add_group_user(group1['id'], user2['id'])
        self.admin_groups_client.add_group_user(group2['id'], user1['id'])
        self.admin_groups_client.add_group_user(group2['id'], user2['id'])
        # user can remove a user in own domain from a group in own domain
        self.do_request('delete_group_user', expected_status=204,
                        group_id=group1['id'], user_id=user1['id'])
        # user cannot remove a user in another domain from a group in own
        # domain
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id=user2['id'])
        # user cannot remove a user in own domain from a group in another
        # domain
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user1['id'])
        # user cannot remove a user in another domain from a group in another
        # domain
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user2['id'])
        # user gets a 403 for nonexistent group
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id='fakegroup', user_id=user1['id'])
        # user gets a 403 for nonexistent user
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id='fakeuser')

    def test_identity_check_user_in_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        self.admin_groups_client.add_group_user(group1['id'], user1['id'])
        self.admin_groups_client.add_group_user(group1['id'], user2['id'])
        self.admin_groups_client.add_group_user(group2['id'], user1['id'])
        self.admin_groups_client.add_group_user(group2['id'], user2['id'])
        # user can check if a user in own domain is in a group in own domain
        self.do_request('check_group_user_existence', expected_status=204,
                        group_id=group1['id'], user_id=user1['id'])
        # user can check if a user in another domain is in a group in own
        # domain
        self.do_request('check_group_user_existence',
                        expected_status=204,
                        group_id=group1['id'], user_id=user2['id'])
        # user cannot check if a user in own domain is in a group in another
        # domain
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user1['id'])
        # user cannot check if a user in another domain is in a group in
        # another domain
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user2['id'])
        # user gets a 403 for nonexistent group
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.Forbidden,
                        group_id='fakegroup', user_id=user1['id'])
        # user gets a 403 for nonexistent user
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id='fakeuser')


class DomainMemberTests(IdentityV3RbacGroupTest, base.BaseIdentityTest):

    credentials = ['domain_member', 'system_admin']

    def test_identity_create_group(self):
        # user cannot create group in own domain
        self.do_request('create_group',
                        expected_status=exceptions.Forbidden,
                        **self.group(domain_id=self.own_domain))
        # user cannot create group in another domain
        self.do_request('create_group',
                        expected_status=exceptions.Forbidden,
                        **self.group(domain_id=self.other_domain))

    def test_identity_get_group(self):
        group = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        # user can get group in own domain
        self.do_request('show_group', group_id=group['id'])
        # user cannot get group in other domain
        group = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        self.do_request('show_group', expected_status=exceptions.Forbidden,
                        group_id=group['id'])
        # user gets a 403 for nonexistent group
        self.do_request('show_group', expected_status=exceptions.Forbidden,
                        group_id='fakegroup')

    def test_identity_list_groups(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        resp = self.do_request('list_groups')
        # user can get groups in own domain
        self.assertIn(group1['id'], set(g['id'] for g in resp['groups']))
        # user cannot get groups in other domain
        self.assertNotIn(group2['id'], set(g['id'] for g in resp['groups']))

    def test_identity_list_groups_for_user(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.other_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        self.admin_groups_client.add_group_user(group1['id'], user1['id'])
        self.admin_groups_client.add_group_user(group1['id'], user2['id'])
        self.admin_groups_client.add_group_user(group2['id'], user1['id'])
        self.admin_groups_client.add_group_user(group2['id'], user2['id'])
        resp = self.do_request('list_user_groups', client=self.users_client,
                               user_id=user1['id'])
        # user can list groups in own domain for user in own domain
        self.assertIn(group1['id'], set(g['id'] for g in resp['groups']))
        # user cannot list groups in other domain for user in own domain
        self.assertNotIn(group2['id'], set(g['id'] for g in resp['groups']))
        # user cannot list groups for user in other domain
        resp = self.do_request('list_user_groups', client=self.users_client,
                               expected_status=exceptions.Forbidden,
                               user_id=user2['id'])
        # user gets a 403 for nonexistent user
        self.do_request('list_user_groups', client=self.users_client,
                        expected_status=exceptions.Forbidden,
                        user_id='fakeuser')

    def test_identity_update_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        # user cannot update a group in own domain
        group_update = {
            'group_id': group1['id'],
            'description': data_utils.arbitrary_string
        }
        self.do_request('update_group', expected_status=exceptions.Forbidden,
                        **group_update)
        # user cannot update a group in other domain
        group_update = {
            'group_id': group2['id'],
            'description': data_utils.arbitrary_string
        }
        self.do_request('update_group', expected_status=exceptions.Forbidden,
                        **group_update)
        # user gets a 403 for nonexistent group
        group_update = {
            'group_id': 'fakegroup',
            'description': data_utils.arbitrary_string
        }
        self.do_request('update_group', expected_status=exceptions.Forbidden,
                        **group_update)

    def test_identity_delete_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        # user cannot delete a group in own domain
        self.do_request('delete_group', expected_status=exceptions.Forbidden,
                        group_id=group1['id'])
        # user cannot delete a group in other domain
        self.do_request('delete_group', expected_status=exceptions.Forbidden,
                        group_id=group2['id'])
        # user gets a 404 for nonexistent group
        self.do_request('delete_group', expected_status=exceptions.NotFound,
                        group_id='fakegroup')

    def test_identity_list_users_in_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.other_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        self.admin_groups_client.add_group_user(group1['id'], user1['id'])
        self.admin_groups_client.add_group_user(group1['id'], user2['id'])
        self.admin_groups_client.add_group_user(group2['id'], user1['id'])
        self.admin_groups_client.add_group_user(group2['id'], user2['id'])
        resp = self.do_request('list_group_users', group_id=group1['id'])
        # user can list users in own domain for group in own domain
        self.assertIn(user1['id'], set(u['id'] for u in resp['users']))
        # user cannot list users in another domain for group in own domain
        self.assertNotIn(user2['id'], set(u['id'] for u in resp['users']))
        # user cannot list users for group in another domain
        self.do_request('list_group_users',
                        expected_status=exceptions.Forbidden,
                        group_id=group2['id'])

    def test_identity_add_user_to_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.other_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        # user cannot add a user in own domain to a group in own domain
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id=user1['id'])
        # user cannot add a user in another domain to a group in own domain
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id=user2['id'])
        # user cannot add a user in own domain to a group in another domain
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user1['id'])
        # user cannot add a user in another domain to a group in another domain
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user2['id'])
        # user gets a 403 for nonexistent group
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id='fakegroup', user_id=user1['id'])
        # user gets a 403 for nonexistent user
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id='fakeuser')

    def test_identity_remove_user_from_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.other_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        self.admin_groups_client.add_group_user(group1['id'], user1['id'])
        self.admin_groups_client.add_group_user(group1['id'], user2['id'])
        self.admin_groups_client.add_group_user(group2['id'], user1['id'])
        self.admin_groups_client.add_group_user(group2['id'], user2['id'])
        # user cannot remove a user in own domain from a group in own domain
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id=user1['id'])
        # user cannot remove a user in another domain from a group in own
        # domain
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id=user2['id'])
        # user cannot remove a user in own domain from a group in another
        # domain
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user1['id'])
        # user cannot remove a user in another domain from a group in another
        # domain
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user2['id'])
        # user gets a 403 for nonexistent group
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id='fakegroup', user_id=user1['id'])
        # user gets a 403 for nonexistent user
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id='fakeuser')

    def test_identity_check_user_in_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        self.admin_groups_client.add_group_user(group1['id'], user1['id'])
        self.admin_groups_client.add_group_user(group1['id'], user2['id'])
        self.admin_groups_client.add_group_user(group2['id'], user1['id'])
        self.admin_groups_client.add_group_user(group2['id'], user2['id'])
        # user can check if a user in own domain is in a group in own domain
        self.do_request('check_group_user_existence', expected_status=204,
                        group_id=group1['id'], user_id=user1['id'])
        # user can check if a user in another domain is in a group in own
        # domain
        self.do_request('check_group_user_existence',
                        expected_status=204,
                        group_id=group1['id'], user_id=user2['id'])
        # user cannot check if a user in own domain is in a group in another
        # domain
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user1['id'])
        # user cannot check if a user in another domain is in a group in
        # another domain
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user2['id'])
        # user gets a 403 for nonexistent group
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.Forbidden,
                        group_id='fakegroup', user_id=user1['id'])
        # user gets a 403 for nonexistent user
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id='fakeuser')


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(IdentityV3RbacGroupTest, base.BaseIdentityTest):

    credentials = ['project_manager', 'system_admin']

    def test_identity_create_group(self):
        # user cannot create group in own domain
        self.do_request('create_group', expected_status=exceptions.Forbidden,
                        **self.group(domain_id=self.own_domain))
        # user cannot create group in another domain
        self.do_request('create_group',
                        expected_status=exceptions.Forbidden,
                        **self.group(domain_id=self.other_domain))

    def test_identity_get_group(self):
        group = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        # user cannot get group in own domain
        self.do_request('show_group', expected_status=exceptions.Forbidden,
                        group_id=group['id'])
        # user cannot get group in other domain
        group = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group['id'])
        self.do_request('show_group', expected_status=exceptions.Forbidden,
                        group_id=group['id'])
        # user gets a 403 for nonexistent group
        self.do_request('show_group', expected_status=exceptions.Forbidden,
                        group_id='fakegroup')

    def test_identity_list_groups(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        self.do_request('list_groups', expected_status=exceptions.Forbidden)

    def test_identity_list_groups_for_user(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.other_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        self.admin_groups_client.add_group_user(group1['id'], user1['id'])
        self.admin_groups_client.add_group_user(group1['id'], user2['id'])
        self.admin_groups_client.add_group_user(group2['id'], user1['id'])
        self.admin_groups_client.add_group_user(group2['id'], user2['id'])
        # user cannot list groups for user in own domain
        self.do_request('list_user_groups', client=self.users_client,
                        expected_status=exceptions.Forbidden,
                        user_id=user1['id'])
        # user cannot list groups for user in other domain
        self.do_request('list_user_groups', client=self.users_client,
                        expected_status=exceptions.Forbidden,
                        user_id=user2['id'])
        # user gets a 403 for nonexistent user
        self.do_request('list_user_groups', client=self.users_client,
                        expected_status=exceptions.Forbidden,
                        user_id='fakeuser')

    def test_identity_update_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        # user cannot update a group in own domain
        group_update = {
            'group_id': group1['id'],
            'description': data_utils.arbitrary_string
        }
        self.do_request('update_group', expected_status=exceptions.Forbidden,
                        **group_update)
        # user cannot update a group in other domain
        group_update = {
            'group_id': group2['id'],
            'description': data_utils.arbitrary_string
        }
        self.do_request('update_group', expected_status=exceptions.Forbidden,
                        **group_update)
        # user gets a 403 for nonexistent group
        group_update = {
            'group_id': 'fakegroup',
            'description': data_utils.arbitrary_string
        }
        self.do_request('update_group', expected_status=exceptions.Forbidden,
                        **group_update)

    def test_identity_delete_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        # user cannot delete a group in own domain
        self.do_request('delete_group', expected_status=exceptions.Forbidden,
                        group_id=group1['id'])
        # user cannot delete a group in other domain
        self.do_request('delete_group', expected_status=exceptions.Forbidden,
                        group_id=group2['id'])
        # user gets a 403 for nonexistent group
        self.do_request('delete_group', expected_status=exceptions.NotFound,
                        group_id='fakegroup')

    def test_identity_list_users_in_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(domain_id=self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.other_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        self.admin_groups_client.add_group_user(group1['id'], user1['id'])
        self.admin_groups_client.add_group_user(group1['id'], user2['id'])
        self.admin_groups_client.add_group_user(group2['id'], user1['id'])
        self.admin_groups_client.add_group_user(group2['id'], user2['id'])
        # user cannot list users for group in own domain
        self.do_request('list_group_users',
                        expected_status=exceptions.Forbidden,
                        group_id=group1['id'])
        # user cannot list users for group in another domain
        self.do_request('list_group_users',
                        expected_status=exceptions.Forbidden,
                        group_id=group2['id'])

    def test_identity_add_user_to_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        # user cannot add a user in own domain to a group in own domain
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id=user1['id'])
        # user cannot add a user in another domain to a group in own domain
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id=user2['id'])
        # user cannot add a user in own domain to a group in another domain
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user1['id'])
        # user cannot add a user in another domain to a group in another domain
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user2['id'])
        # user gets a 403 for nonexistent group
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id='fakegroup', user_id=user1['id'])
        # user gets a 403 for nonexistent user
        self.do_request('add_group_user', expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id='fakeuser')

    def test_identity_remove_user_from_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        self.admin_groups_client.add_group_user(group1['id'], user1['id'])
        self.admin_groups_client.add_group_user(group1['id'], user2['id'])
        self.admin_groups_client.add_group_user(group2['id'], user1['id'])
        self.admin_groups_client.add_group_user(group2['id'], user2['id'])
        # user cannot remove a user in own domain from a group in own domain
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id=user1['id'])
        # user cannot remove a user in another domain from a group in own
        # domain
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id=user2['id'])
        # user cannot remove a user in own domain from a group in another
        # domain
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user1['id'])
        # user cannot remove a user in another domain from a group in another
        # domain
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user2['id'])
        # user gets a 403 for nonexistent group
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id='fakegroup', user_id=user1['id'])
        # user gets a 403 for nonexistent user
        self.do_request('delete_group_user',
                        expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id='fakeuser')

    def test_identity_check_user_in_group(self):
        group1 = self.admin_groups_client.create_group(
            **self.group(self.own_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group1['id'])
        group2 = self.admin_groups_client.create_group(
            **self.group(self.other_domain))['group']
        self.addCleanup(self.admin_groups_client.delete_group, group2['id'])
        user1 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user1['id'])
        user2 = self.admin_client.users_v3_client.create_user(
            name=data_utils.rand_name('user'),
            domain_id=self.own_domain)['user']
        self.addCleanup(self.admin_client.users_v3_client.delete_user,
                        user2['id'])
        self.admin_groups_client.add_group_user(group1['id'], user1['id'])
        self.admin_groups_client.add_group_user(group1['id'], user2['id'])
        self.admin_groups_client.add_group_user(group2['id'], user1['id'])
        self.admin_groups_client.add_group_user(group2['id'], user2['id'])
        # user cannot check if a user in own domain is in a group in own domain
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id=user1['id'])
        # user cannot check if a user in another domain is in a group in own
        # domain
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id=user2['id'])
        # user cannot check if a user in own domain is in a group in another
        # domain
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user1['id'])
        # user cannot check if a user in another domain is in a group in
        # another domain
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.Forbidden,
                        group_id=group2['id'], user_id=user2['id'])
        # user gets a 403 for nonexistent group
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.Forbidden,
                        group_id='fakegroup', user_id=user1['id'])
        # user gets a 403 for nonexistent user
        self.do_request('check_group_user_existence',
                        expected_status=exceptions.Forbidden,
                        group_id=group1['id'], user_id='fakeuser')


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']



# Copyright 2020 SUSE LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from keystone_tempest_plugin.tests.rbac.v3 import base as rbac_base


class IdentityV3RbacImpliedRoleTest(rbac_base.IdentityV3RbacBaseTests,
                                    metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(IdentityV3RbacImpliedRoleTest, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.roles_v3_client
        cls.admin_client = cls.os_system_admin
        cls.admin_roles_client = cls.admin_client.roles_v3_client

    @classmethod
    def resource_setup(cls):
        super(IdentityV3RbacImpliedRoleTest, cls).resource_setup()
        cls.prior_role = cls.admin_roles_client.create_role(
            name=data_utils.rand_name('prior_role'))['role']['id']
        cls.addClassResourceCleanup(
            cls.admin_roles_client.delete_role, cls.prior_role)
        cls.implied_role = cls.admin_roles_client.create_role(
            name=data_utils.rand_name('implied_role'))['role']['id']
        cls.addClassResourceCleanup(
            cls.admin_roles_client.delete_role, cls.implied_role)

    @abc.abstractmethod
    def test_identity_create_implied_role(self):
        """Test identity:create_implied_role policy.

        This test must check:
          * whether the persona can create an implied role
        """
        pass

    @abc.abstractmethod
    def test_identity_get_implied_role(self):
        """Test identity:get_implied_role policy.

        This test must check:
          * whether the persona can get an implied role
        """
        pass

    @abc.abstractmethod
    def test_identity_list_implied_roles(self):
        """Test identity:list_implied_roles policy.

        This test must check:
          * whether the persona can list implied roles
        """
        pass

    @abc.abstractmethod
    def test_identity_list_role_inference_rules(self):
        """Test identity:list_role_inference_rules policy.

        This test must check:
          * whether the persona can list role inference rules
        """
        pass

    @abc.abstractmethod
    def test_identity_delete_implied_role(self):
        """Test identity:delete_implied_role policy.

        This test must check
          * whether the persona can delete an implied role
        """
        pass

    @abc.abstractmethod
    def test_identity_check_implied_role(self):
        """Test identity:check_implied_role policy.

        This test must check:
          * whether the persona can check an association between two roles
        """
        pass


class SystemAdminTests(IdentityV3RbacImpliedRoleTest, base.BaseIdentityTest):

    credentials = ['system_admin']

    def test_identity_create_implied_role(self):
        self.do_request('create_role_inference_rule',
                        expected_status=201,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)
        self.addCleanup(self.admin_roles_client.delete_role_inference_rule,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)

    def test_identity_get_implied_role(self):
        self.admin_roles_client.create_role_inference_rule(
            prior_role=self.prior_role, implies_role=self.implied_role)
        self.addCleanup(self.admin_roles_client.delete_role_inference_rule,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)
        self.do_request('show_role_inference_rule',
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)

    def test_identity_list_implied_roles(self):
        self.admin_roles_client.create_role_inference_rule(
            prior_role=self.prior_role, implies_role=self.implied_role)
        self.addCleanup(self.admin_roles_client.delete_role_inference_rule,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)
        self.do_request('list_role_inferences_rules',
                        prior_role=self.prior_role)

    def test_identity_list_role_inference_rules(self):
        self.admin_roles_client.create_role_inference_rule(
            prior_role=self.prior_role, implies_role=self.implied_role)
        self.addCleanup(self.admin_roles_client.delete_role_inference_rule,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)
        self.do_request('list_all_role_inference_rules')

    def test_identity_delete_implied_role(self):
        self.admin_roles_client.create_role_inference_rule(
            prior_role=self.prior_role, implies_role=self.implied_role)
        self.do_request('delete_role_inference_rule',
                        expected_status=204,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)

    def test_identity_check_implied_role(self):
        self.admin_roles_client.create_role_inference_rule(
            prior_role=self.prior_role, implies_role=self.implied_role)
        self.addCleanup(self.admin_roles_client.delete_role_inference_rule,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)
        self.do_request('check_role_inference_rule',
                        expected_status=204,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)


class SystemMemberTests(SystemAdminTests):

    credentials = ['system_member', 'system_admin']

    def test_identity_create_implied_role(self):
        self.do_request('create_role_inference_rule',
                        expected_status=exceptions.Forbidden,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)

    def test_identity_delete_implied_role(self):
        self.admin_roles_client.create_role_inference_rule(
            prior_role=self.prior_role, implies_role=self.implied_role)
        self.addCleanup(self.admin_roles_client.delete_role_inference_rule,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)
        self.do_request('delete_role_inference_rule',
                        expected_status=exceptions.Forbidden,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)


class SystemReaderTests(SystemMemberTests):

    credentials = ['system_reader', 'system_admin']


class DomainAdminTests(SystemReaderTests):

    credentials = ['domain_admin', 'system_admin']

    def test_identity_get_implied_role(self):
        self.admin_roles_client.create_role_inference_rule(
            prior_role=self.prior_role, implies_role=self.implied_role)
        self.addCleanup(self.admin_roles_client.delete_role_inference_rule,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)
        self.do_request('show_role_inference_rule',
                        expected_status=exceptions.Forbidden,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)

    def test_identity_list_implied_roles(self):
        self.admin_roles_client.create_role_inference_rule(
            prior_role=self.prior_role, implies_role=self.implied_role)
        self.addCleanup(self.admin_roles_client.delete_role_inference_rule,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)
        self.do_request('list_role_inferences_rules',
                        expected_status=exceptions.Forbidden,
                        prior_role=self.prior_role)

    def test_identity_list_role_inference_rules(self):
        self.admin_roles_client.create_role_inference_rule(
            prior_role=self.prior_role, implies_role=self.implied_role)
        self.addCleanup(self.admin_roles_client.delete_role_inference_rule,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)
        self.do_request('list_all_role_inference_rules',
                        expected_status=exceptions.Forbidden)

    def test_identity_check_implied_role(self):
        self.admin_roles_client.create_role_inference_rule(
            prior_role=self.prior_role, implies_role=self.implied_role)
        self.addCleanup(self.admin_roles_client.delete_role_inference_rule,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)
        self.do_request('check_role_inference_rule',
                        expected_status=exceptions.Forbidden,
                        prior_role=self.prior_role,
                        implies_role=self.implied_role)


class DomainManagerTests(DomainAdminTests):

    credentials = ['domain_manager', 'system_admin']


class DomainMemberTests(DomainManagerTests):

    credentials = ['domain_member', 'system_admin']


class DomainReaderTests(DomainMemberTests):

    credentials = ['domain_reader', 'system_admin']


class ProjectAdminTests(SystemAdminTests):

    credentials = ['project_admin', 'system_admin']


class ProjectManagerTests(DomainReaderTests):

    credentials = ['project_manager', 'system_admin']


class ProjectMemberTests(ProjectManagerTests):

    credentials = ['project_member', 'system_admin']


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'system_admin']
